package mmexample2;

// You will need these new imports
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Scanner;
import javafx.stage.FileChooser;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import mm.*;

public class MartianManagerGui2ToClass extends Application {
	
	// Read/Write events
	protected Button btnReadMartiansBasic, btnWriteMartians, btnReadMartiansRobust;
	// Results
	protected TextArea txaMessage;

	// Path to files.
	// You will need to change this. Probably it will be: "src/application"
	private final String PATH = "src/mmexample2";
	
	// Domain variables
	MartianManager mm = new MartianManager();
	ArrayList<Martian> invaders = new ArrayList<>();

	@Override
	public void start(Stage primaryStage) {
		try {
			// Notice that this method requires the "primaryStage" as an argument. This is because the FileChooser
			// requires a stage to display the dialog. Since you already have a "buildGui" method, you will need 
			// to modify your code accordingly.
			Pane root = buildGui(primaryStage);
			Scene scene = new Scene(root,620,325);
			primaryStage.setTitle("Example-Read/Write");
			scene.getStylesheets().add(getClass().getResource("application.css").toExternalForm());
			primaryStage.setScene(scene);
			primaryStage.show();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Notice that this method accepts a "stage" as an argument. This is because the FileChooser
	 * requires a stage to display the dialog. Since you already have a "buildGui" method,
	 * you will need to modify your code accordingly.
	 */
	private Pane buildGui(Stage stage) {
		// Build components
		Pane p1 = buildReadWrite(stage);
		Pane p2 = buildResults();
		// Build root container
		VBox root = new VBox();
		root.getChildren().addAll(p1,p2);
		return root;
	}

	private Pane buildReadWrite(Stage stage) {
		// Create buttons
		btnReadMartiansBasic = new Button("Read-Basic");
		// Notice this event handler requires the "stage."
		btnReadMartiansBasic.setOnAction(new ReadMartiansBasicEventHandler(stage));
		btnWriteMartians = new Button("Write");
		btnWriteMartians.setOnAction(new WriteMartiansEventHandler(stage));
		btnReadMartiansRobust = new Button("Read-Robust");
		btnReadMartiansRobust.setDisable(true);
		// Build container, style, and add components.
		HBox hBox = new HBox();
		hBox.getStyleClass().add("boxBorder");
		hBox.getChildren().addAll(btnReadMartiansBasic, btnWriteMartians, btnReadMartiansRobust);
		
		return hBox;
	}

	// You already something similar to this in your code.
	private Pane buildResults() {
		Label lbl = new Label("Results");
		txaMessage = new TextArea();
		VBox vBox = new VBox();
		vBox.getChildren().addAll(lbl, txaMessage);
		return vBox;
	}

	//------------------------------------------------------------------------------------------
	// Event Handlers
	//------------------------------------------------------------------------------------------

	/**
	 *  Handles when the "Read-Basic" button is pressed. It requires
	 *  a constructor so that it can remember the "stage" as an instance variable.
	 */
	private class ReadMartiansBasicEventHandler implements EventHandler<ActionEvent> {
		Stage stage;
		public ReadMartiansBasicEventHandler(Stage stage) {
			super();
			this.stage = stage;
		}
		/**
		 * Displays the standard "Open" dialog so the user can choose the
		 * file to read. Note that it returns a File object, so you don't
		 * need to create another one. In case you are interested, the getFile 
		 * method is below. 
		 */
		@Override
		public void handle(ActionEvent event) {
			File file = getFile(stage, "Open"); // Display Open dialog
			if( file != null) {
	            try {
	            	// Call this method to do the actual reading 
					readMartiansFile(file);
				}
	            catch (FileNotFoundException e) {
					txaMessage.setText("Error reading file");
					e.printStackTrace();
				}
			}
		}
	}
	
	/**
	 *  Handles when the "Write" button is pressed.
	 */
	private class WriteMartiansEventHandler implements EventHandler<ActionEvent> {
		Stage stage;
		public WriteMartiansEventHandler(Stage stage) {
			super();
			this.stage = stage;
		}
		/**
		 * Displays the standard "Save" dialog so the user can type a file name or choose an
		 * existing file to overwrite. Note that it returns a File object, and passes it to
		 * the "writeMartiansFile" which you will write. In case you are interested, 
		 * the getFile method is below. 
		 */
		@Override
		public void handle(ActionEvent event) {
			File file = getFile(stage, "Save"); // Display Save dialog
			if( file != null) {
	            try {
	            	// YOU WRITE THIS METHOD (stub below) 
					writeMartiansFile(file);
				}
	            catch (FileNotFoundException e) {
					txaMessage.setText("Error writing file");
					e.printStackTrace();
				}
			}
		}
	}
	
	//------------------------------------------------------------------------------------------
	// Methods you will write
	//------------------------------------------------------------------------------------------
	
	/**
	 * ---> YOU WRITE THIS METHOD. <--
	 * This method should read the martians from the "file" that was input.
	 */
	protected void readMartiansFile(File file) throws FileNotFoundException {
		Scanner input = new Scanner(file);
		txaMessage.setText("I just pretended to read the file: " + file.getName() + "\n");
		input.close();
	}

	/**
	 * ---> YOU WRITE THIS METHOD. <--
	 * This method should write the martians in the MartianManager to the "file" that was input.
	 */
	protected void writeMartiansFile(File file) throws FileNotFoundException {
		PrintWriter writer = new PrintWriter(file);
		txaMessage.setText("I just pretended to write to the file: " + file.getName() + "\n");
		writer.close();
	}
	
	//------------------------------------------------------------------------------------------
	// Helper methods
	//------------------------------------------------------------------------------------------

	/**
	 *  Display FileChooser to select an input or output file
	 */
	private File getFile(Stage stage, String type) {
	    FileChooser fileChooser = configureFileChooser();
	    File file = null;
	    
	 	switch(type) {
	 	case "Open":
	        file = fileChooser.showOpenDialog(stage);
	 		break;
	 	case "Save":
	        file = fileChooser.showSaveDialog(stage);
	 		break;
	 	}
	 	return file;
	}

	/**
	 * Configure FileChooser
	 */
	private FileChooser configureFileChooser() {
		FileChooser fileChooser = new FileChooser();
	    FileChooser.ExtensionFilter extFilter = new FileChooser.ExtensionFilter("TXT files (*.txt)", "*.txt");
	    fileChooser.getExtensionFilters().add(extFilter);
	    File initPath = new File(PATH);
	    fileChooser.setInitialDirectory(initPath);
		return fileChooser;
	}

	public static void main(String[] args) {
		launch(args);
	}
}
